<?php

/**
 * @file
 * Contains the CommerceGooglePayInterface interface.
 */

use Stripe\PaymentIntent;
use Stripe\Stripe;

/**
 * Interface representing Commerce Stripe Payment Intent Payment Request backend class.
 */
interface CommerceStripePiPaymentRequestInterface {

  /**
   * CommerceStripePaymentRequestInterface constructor.
   *
   * @param $order
   *   Commerce order.
   */
  public function __construct($order);

  /**
   * Make and finalize payment transaction.
   *
   * @throws Exception
   */
  public function makeTransaction();

}

/**
 * Implementation of one-time Stripe Payment Intent Payment Request.
 */
class CommerceStripePiPaymentRequest implements CommerceStripePiPaymentRequestInterface {

  protected $order;

  protected $context = [];

  /**
   * CommerceStripePaymentRequest constructor.
   *
   * @param $order
   *   Commerce order.
   */
  public function __construct($order) {
    $this->order = $order;
    $this->context = $this->initGlobalContext();
  }


  /**
   * Get all required data from global scope.
   *
   * @throws Exception
   */
  protected function initGlobalContext() {
    // Include Stripe library.
    $library = libraries_get_path('stripe-php');
    require_once($library . '/init.php');

    $context = [
      'payment_request' => [],
      'payment_method' => [],
      'stripe_data' => [],
      'metadata' => [],
    ];
    // Parse the data from $_POST.
    $post = json_decode(file_get_contents('php://input'), TRUE);
    // Get stripe backend key and init it.
    $payment_method = commerce_payment_method_instance_load('commerce_stripe_pi_payment_request|commerce_payment_commerce_stripe_pi_payment_request');

    $context['payment_request'] = isset($post['data']['payment_request'])?$post['data']['payment_request']:"";
    $context['payment_method'] = $payment_method;
    $context['payment_method_remote_id'] = isset($post['data']['payment_method_remote_id'])?$post['data']['payment_method_remote_id']:"";
    $context['stripe_data'] = isset($post['data']['result'])?$post['data']['result']: array();
    $context['metadata'] = ["order_id" => $this->order->order_id];

    // Allow other modules to alter data / update order before the transaction.
    drupal_alter('commerce_stripe_pi_payment_request_stripe_context', $this->order, $context);

    // Reload the order after possible changes in alter hooks.
    $this->order = commerce_order_load($this->order->order_id);

    // Init Stripe.
    $stripe_secret_key = $context['payment_method']['settings']['stripe_secret_key'];
    Stripe::setApiKey($stripe_secret_key);

    return $context;
  }

  /**
   * Helper to get payment setting from context array.
   *
   * @param string $setting_name
   *   Setting name.
   *
   * @return mixed
   *   Requested setting.
   */
  public function getPaymentSetting($setting_name) {
    return $this->context['payment_method']['settings'][$setting_name];
  }

  /**
   * Make single transaction.
   *
   * @throws \Stripe\Error\Base
   */
  public function makeTransaction() {
    $order = $this->order;
    $context = $this->context;

    $order_wrapper = entity_metadata_wrapper('commerce_order', $this->order);
    $order_total_data = $order_wrapper->commerce_order_total->value();
    $payment_method_id = $context['payment_method']['instance_id'];

    $amount = $order_total_data['amount'];
    //get exact transaction data that is being sent so that we have correct one
    // on capture and void and refunds
    $payment_method = commerce_payment_method_instance_load($payment_method_id);
    if($order_total_data['currency_code'] != $payment_method['settings']['currency']){
      $amount = (int) commerce_currency_convert($order_total_data['amount'], $order_total_data['currency_code'], $payment_method['settings']['currency']);
    }

    try {
      // Payment intent.
      $intent = $this->getPaymentIntent();
      $transaction = commerce_payment_transaction_new('commerce_stripe_pi_payment_request', $order->order_id);
      $transaction->instance_id = $payment_method_id;
      $transaction->amount = $amount;
      $transaction->currency_code = $payment_method['settings']['currency'];
      $transaction->payload[REQUEST_TIME] = $intent->getLastResponse()->json;
      $transaction->remote_id = $intent->id;
      $transaction->remote_status = $intent->status;
      if($intent->status != 'requires_action' || $intent->status != 'requires_source_action') {

        if ($intent->status === 'requires_capture' ||
          $intent->status == 'succeeded') {
          if ($intent->status === 'succeeded') {
            $transaction->status = COMMERCE_PAYMENT_STATUS_SUCCESS;
            $transaction->remote_status = $intent->status;
          }
          elseif ($intent->status === 'requires_capture') {
            $transaction->status = COMMERCE_PAYMENT_STATUS_PENDING;
            $transaction->remote_status = 'AUTH_ONLY';
          }
          // Prepare a transaction object to log the API response.
          $transaction->message = t('Stripe Payment Intent created successfully.');
          commerce_payment_transaction_save($transaction);

          // Finalize order.
          commerce_order_status_update($this->order, 'checkout_complete', FALSE, FALSE);
          commerce_checkout_complete($this->order);
        }
        elseif ($intent->status == 'requires_payment_method' ||
          $intent->status == 'requires_source') {
          // empty the payment intent.
          $transaction->status = COMMERCE_PAYMENT_STATUS_FAILURE;
          $transaction->message = t('Could not complete Stripe Payment Intent.').
            commerce_payment_transaction_save($transaction);
          commerce_order_status_update($this->order, 'checkout_review', FALSE, FALSE);
        }
      }

    } catch (\Stripe\Error\Base $e) {
      $transaction = commerce_payment_transaction_new('commerce_stripe_pi_payment_request', $order->order_id);
      $transaction->instance_id = $payment_method_id;
      $transaction->amount = $amount;
      $transaction->currency_code = $payment_method['settings']['currency'];
      $transaction->payload[REQUEST_TIME] = $e->getJsonBody();
      $transaction->remote_status = 'error';
      $transaction->status = COMMERCE_PAYMENT_STATUS_FAILURE;
      $transaction->message = 'Could not complete Stripe Payment Intent. ' . $e->getMessage();
      commerce_payment_transaction_save($transaction);

      watchdog_exception('commerce_stripe_pi_payment_request', $e);
      // Server will return 500 to the frontend.
      drupal_set_message(t('An error occurred processing this payment. The error is: @e', [
        '@e' => $e,
      ]), 'error');
    }

    if ($intent->status == 'requires_confirmation' ||
      $intent->status == 'requires_source_action' ||
      $intent->status == 'requires_action'||
      $intent->status == 'requires_payment_method'
      || $intent->status == 'requires_source') {
      # Tell the client to handle the action
      $response = [
        'requires_action' => TRUE,
        'payment_intent_client_secret' => $intent->client_secret,
        'payment_method_remote_id' => $context['payment_method_remote_id'],
        'redirect_uri' => url(commerce_checkout_order_uri($this->order)),
      ];
    }
    else {
      if ($intent->status == 'succeeded' || $intent->status == 'requires_capture') {
        # The payment didn’t need any additional actions and completed!
        # Handle post-payment fulfillment
        $response = [
          'success' => TRUE,
          'redirect_uri' => url(commerce_checkout_order_uri($this->order)),
        ];
      }
      else {
        # Invalid status
        //http_response_code(500);
        $response = ['error' => 'Invalid PaymentIntent status'];
      }
    }
    return $response;
  }

  /**
   * Get current payment intent.
   *
   * @return \Stripe\PaymentIntent
   */
  public function getPaymentIntent() {
    $order = $this->order;
    $context = $this->context;

    $order_wrapper = entity_metadata_wrapper('commerce_order', $this->order);
    $order_total_data = $order_wrapper->commerce_order_total->value();

    // Check if we already have payment intent
    $payment_intent_array = _commerce_stripe_pi_payment_request_get_payment_intent_order($order);

    if (empty($payment_intent_array) ||
      ($payment_intent_array['status'] === 'requires_confirmation' ||
        $payment_intent_array['status'] === 'requires_payment_method')) {

      $values_pi = [
        'payment_method' => $context['payment_method_remote_id'],
        'amount' => $order_total_data['amount'],
        'currency' => $context['payment_method']['settings']['currency'],
        'payment_method_types' => ['card'],
        'description' => $context['payment_request']['total']['label'],
        'metadata' => $context['metadata'],
        'capture_method' => $context['payment_method']['settings']['txn_type'] === 'auth_capture' ? 'automatic' : 'manual',
        'confirm' => TRUE,
      ];
      // Allow other modules to alter payment intent data.
      drupal_alter('commerce_stripe_pi_payment_request_payment_intent_data', $values_pi, $order_wrapper, $context);
      try{

        // Create payment intent if empty.
        $intent = PaymentIntent::create($values_pi);
        // Save payment intent.
        $payment_intent = json_decode(json_encode($intent), TRUE);
        _commerce_stripe_pi_payment_request_set_payment_intent_order($order, $payment_intent);
      }catch (\Stripe\Error\Base $e){
        watchdog('stripe_payment_request',$e->getMessage());
      }
      catch (\Stripe\Exception\ApiErrorException $e) {
        watchdog('stripe_payment_request',$e->getMessage());
      }
    }
    else {
      $intent = PaymentIntent::retrieve($payment_intent_array['id']);
    }
    return $intent;
  }

}
